@@ -1,22 +1,21 @@
 
 
-/*! @file 
- * \brief Solves a system of linear equations A*X=B
- *
- * <pre>
- * -- Distributed SuperLU routine (version 2.2) --
- * Lawrence Berkeley National Lab, Univ. of California Berkeley.
- * November 1, 2007
- * Feburary 20, 2008
- * </pre>
- */
-
 #include <math.h>
 #include "superlu_ddefs.h"
 
-/*! \brief
+void
+pdgssvx(superlu_options_t_Distributed *options, SuperMatrix *A, 
+	ScalePermstruct_t *ScalePermstruct,
+	double B[], int ldb, int nrhs, gridinfo_t *grid,
+	LUstruct_t *LUstruct, SOLVEstruct_t *SOLVEstruct, double *berr,
+	SuperLUStat_t *stat, int *info)
+{
+/* 
+ * -- Distributed SuperLU routine (version 2.0) --
+ * Lawrence Berkeley National Lab, Univ. of California Berkeley.
+ * March 15, 2003
+ *
  *
- * <pre>
  * Purpose
  * =======
  *
@@ -27,34 +26,10 @@
  * Static pivoting is a technique that combines the numerical stability
  * of partial pivoting with the scalability of Cholesky (no pivoting),
  * to run accurately and efficiently on large numbers of processors.
+ *
  * See our paper at http://www.nersc.gov/~xiaoye/SuperLU/ for a detailed
  * description of the parallel algorithms.
  *
- * The input matrices A and B are distributed by block rows.
- * Here is a graphical illustration (0-based indexing):
- *
- *                        A                B
- *               0 ---------------       ------
- *                   |           |        |  |
- *                   |           |   P0   |  |
- *                   |           |        |  |
- *                 ---------------       ------
- *        - fst_row->|           |        |  |
- *        |          |           |        |  |
- *       m_loc       |           |   P1   |  |
- *        |          |           |        |  |
- *        -          |           |        |  |
- *                 ---------------       ------
- *                   |    .      |        |. |
- *                   |    .      |        |. |
- *                   |    .      |        |. |
- *                 ---------------       ------
- * 
- * where, fst_row is the row number of the first row,
- *        m_loc is the number of rows local to this processor
- * These are defined in the 'SuperMatrix' structure, see supermatrix.h.
- *
- *
  * Here are the options for using this code:
  *
  *   1. Independent of all the other options specified below, the
@@ -117,9 +92,10 @@
  *           later).
  *
  *        o  A, the input matrix A overwritten by the scaled and permuted
- *              matrix diag(R)*A*diag(C)*Pc^T, where 
- *              Pc is the row permutation matrix determined by
- *                  ScalePermstruct->perm_c
+ *              matrix Pc*Pr*diag(R)*A*diag(C), where 
+ *              Pr and Pc are row and columns permutation matrices determined
+ *                  by ScalePermstruct->perm_r and ScalePermstruct->perm_c, 
+ *                  respectively, and 
  *              diag(R) and diag(C) are diagonal scaling matrices determined
  *                  by ScalePermstruct->DiagScale, ScalePermstruct->R and 
  *                  ScalePermstruct->C
@@ -128,7 +104,7 @@
  *
  *                A1 = Pc*Pr*diag(R)*A*diag(C)*Pc^T = L*U
  *
- *               (Note that A1 = Pc*Pr*Aout, where Aout is the matrix stored
+ *               (Note that A1 = Aout * Pc^T, where Aout is the matrix stored
  *                in A on output.)
  *
  *   3. The second value of options->Fact assumes that a matrix with the same
@@ -227,7 +203,8 @@
  *
  *        o  A, the unfactored matrix, only in the case that iterative
  *              refinment is to be done (specifically A must be the output
- *              A from the previous call, so that it has been scaled and permuted)
+ *              A from the previous call, so that it has been scaled and
+ *              permuted)
  *        o  all of ScalePermstruct
  *        o  all of LUstruct, including the actual numerical values of
  *           L and U
@@ -237,7 +214,7 @@
  * Arguments
  * =========
  *
- * options (input) superlu_options_t* (global)
+ * options (input) superlu_options_t_Distributed* (global)
  *         The structure defines the input parameters to control
  *         how the LU decomposition will be performed.
  *         The following fields should be defined for this structure:
@@ -316,6 +293,7 @@
  *           = NATURAL:       natural ordering.
  *           = MMD_AT_PLUS_A: minimum degree ordering on structure of A'+A.
  *           = MMD_ATA:       minimum degree ordering on structure of A'*A.
+ *           = COLAMD:        approximate minimum degree column ordering.
  *           = MY_PERMC:      the ordering given in ScalePermstruct->perm_c.
  *         
  *         o ReplaceTinyPivot (yes_no_t)
@@ -340,12 +318,15 @@
  *           See supermatrix.h for the definition of 'SuperMatrix'.
  *           This routine only handles square A, however, the LU factorization
  *           routine PDGSTRF can factorize rectangular matrices.
- *         On exit, A may be overwtirren by diag(R)*A*diag(C)*Pc^T,
- *           depending on ScalePermstruct->DiagScale and options->ColPerm:
+ *         On exit, A may be overwtirren by Pc*Pr*diag(R)*A*diag(C),
+ *           depending on ScalePermstruct->DiagScale, options->RowPerm and
+ *           options->ColPerm:
  *             if ScalePermstruct->DiagScale != NOEQUIL, A is overwritten by
  *                diag(R)*A*diag(C).
+ *             if options->RowPerm != NATURAL, A is further overwritten by
+ *                Pr*diag(R)*A*diag(C).
  *             if options->ColPerm != NATURAL, A is further overwritten by
- *                diag(R)*A*diag(C)*Pc^T.
+ *                Pc*Pr*diag(R)*A*diag(C).
  *           If all the above condition are true, the LU decomposition is
  *           performed on the matrix Pc*Pr*diag(R)*A*diag(C)*Pc^T.
  *
@@ -478,16 +459,8 @@
  *                failure occurred, plus A->ncol.
  *
  * See superlu_ddefs.h for the definitions of varioous data types.
- * </pre>
+ *
  */
-
-void
-pdgssvx(superlu_options_t *options, SuperMatrix *A, 
-	ScalePermstruct_t *ScalePermstruct,
-	double B[], int ldb, int nrhs, gridinfo_t *grid,
-	LUstruct_t *LUstruct, SOLVEstruct_t *SOLVEstruct, double *berr,
-	SuperLUStat_t *stat, int *info)
-{
     NRformat_loc *Astore;
     SuperMatrix GA;      /* Global A in NC format */
     NCformat *GAstore;
@@ -516,7 +489,7 @@ pdgssvx(superlu_options_t *options, SuperMatrix *A,
     int_t    *rowptr, *colind;  /* Local A in NR*/
     int_t    *rowind_loc, *colptr_loc;
     int_t    colequ, Equil, factored, job, notran, rowequ, need_value;
-    int_t    i, iinfo, j, irow, m, n, nnz, permc_spec;
+    int_t    i, iinfo, j, irow, m, n, nnz, permc_spec, dist_mem_use;
     int_t    nnz_loc, m_loc, fst_row, icol;
     int      iam;
     int      ldx;  /* LDA for matrix X (local). */
@@ -524,35 +497,22 @@ pdgssvx(superlu_options_t *options, SuperMatrix *A,
     double   *C, *R, *C1, *R1, amax, anorm, colcnd, rowcnd;
     double   *X, *b_col, *b_work, *x_col;
     double   t;
-    float    GA_mem_use;    /* memory usage by global A */
-    float    dist_mem_use; /* memory usage during distribution */
-    mem_usage_t num_mem_usage, symb_mem_usage;
+    static mem_usage_t_Distributed num_mem_usage, symb_mem_usage;
 #if ( PRNTlevel>= 2 )
     double   dmin, dsum, dprod;
 #endif
     int_t procs;
 
-    /* Structures needed for parallel symbolic factorization */
-    int_t *sizes, *fstVtxSep, parSymbFact;
-    int   noDomains, nprocs_num;
-    MPI_Comm symb_comm; /* communicator for symbolic factorization */
-    int   col, key; /* parameters for creating a new communicator */
-    Pslu_freeable_t Pslu_freeable;
-    float  flinfo;
-
     /* Initialization. */
-    m       = A->nrow;
-    n       = A->ncol;
-    Astore  = (NRformat_loc *) A->Store;
+    m = A->nrow;
+    n = A->ncol;
+    Astore = (NRformat_loc *) A->Store;
     nnz_loc = Astore->nnz_loc;
-    m_loc   = Astore->m_loc;
+    m_loc = Astore->m_loc;
     fst_row = Astore->fst_row;
-    a       = (double *) Astore->nzval;
-    rowptr  = Astore->rowptr;
-    colind  = Astore->colind;
-    sizes   = NULL;
-    fstVtxSep = NULL;
-    symb_comm = MPI_COMM_NULL;
+    a = Astore->nzval;
+    rowptr = Astore->rowptr;
+    colind = Astore->colind;
 
     /* Test the input parameters. */
     *info = 0;
@@ -584,8 +544,6 @@ pdgssvx(superlu_options_t *options, SuperMatrix *A,
     factored = (Fact == FACTORED);
     Equil = (!factored && options->Equil == YES);
     notran = (options->Trans == NOTRANS);
-    parSymbFact = options->ParSymbFact;
-	
     iam = grid->iam;
     job = 5;
     if ( factored || (Fact == SamePattern_SameRowPerm && Equil) ) {
@@ -607,8 +565,7 @@ pdgssvx(superlu_options_t *options, SuperMatrix *A,
     CHECK_MALLOC(iam, "Enter pdgssvx()");
 #endif
 
-    /* Not factored & ask for equilibration */
-    if ( Equil && Fact != SamePattern_SameRowPerm ) { 
+    if ( Equil ) {
 	/* Allocate storage if not done so before. */
 	switch ( ScalePermstruct->DiagScale ) {
 	    case NOEQUIL:
@@ -673,7 +630,8 @@ pdgssvx(superlu_options_t *options, SuperMatrix *A,
 		}
 	        break;
 	    }
-	} else { /* Compute R & C from scratch */
+	} else {
+
             /* Compute the row and column scalings. */
 	    pdgsequ(A, R, C, &rowcnd, &colcnd, &amax, &iinfo, grid);
 
@@ -704,159 +662,157 @@ pdgssvx(superlu_options_t *options, SuperMatrix *A,
 #endif
     } /* if Equil ... */
 
-    if ( !factored ) { /* Skip this if already factored. */
-        /*
-         * Gather A from the distributed compressed row format to
-         * global A in compressed column format.
-         * Numerical values are gathered only when a row permutation
-         * for large diagonal is sought after.
-         */
-	if ( Fact != SamePattern_SameRowPerm &&
-             (parSymbFact == NO || options->RowPerm != NO) ) {
 
-            need_value = (options->RowPerm == LargeDiag);
+    /*
+     * Gather A from the distributed compressed row format to
+     * global A in compressed column format.
+     * Numerical values are gathered only when a row permutation
+     * for large diagonal is sought after.
+     */
+    need_value = (options->RowPerm == LargeDiag &&
+		  Fact != SamePattern_SameRowPerm && !factored);
+    pdCompRow_loc_to_CompCol_global(need_value, A, grid, &GA);
+    GAstore = (NCformat *) GA.Store;
+    colptr = GAstore->colptr;
+    rowind = GAstore->rowind;
+    nnz = GAstore->nnz;
+    if ( need_value ) a_GA = GAstore->nzval;
+    else assert(GAstore->nzval == NULL);
 
-            pdCompRow_loc_to_CompCol_global(need_value, A, grid, &GA);
 
-            GAstore = (NCformat *) GA.Store;
-            colptr = GAstore->colptr;
-            rowind = GAstore->rowind;
-            nnz = GAstore->nnz;
-            GA_mem_use = (nnz + n + 1) * sizeof(int_t);
+    /* ------------------------------------------------------------
+       Find the row permutation for A.
+       ------------------------------------------------------------*/
+    if ( options->RowPerm != NO ) {
+	t = SuperLU_timer_();
+	if ( options->RowPerm == MY_PERMR ) { /* Use user's perm_r. */
+	    /* Permute the global matrix GA for symbfact() */
+	    for (i = 0; i < colptr[n]; ++i) {
+	        irow = rowind[i]; 
+		rowind[i] = perm_r[irow];
+	    }
+	} else if ( !factored && Fact != SamePattern_SameRowPerm ) {
+	    /* Get a new perm_r[] */
+	    if ( job == 5 ) {
+		/* Allocate storage for scaling factors. */
+		if ( !(R1 = (double *) SUPERLU_MALLOC(m * sizeof(double))) ) 
+		    ABORT("SUPERLU_MALLOC fails for R1[]");
+		if ( !(C1 = (double *) SUPERLU_MALLOC(n * sizeof(double))) )
+		    ABORT("SUPERLU_MALLOC fails for C1[]");
+	    }
 
-            if ( need_value ) {
-                a_GA = (double *) GAstore->nzval;
-                GA_mem_use += nnz * sizeof(double);
-            } else assert(GAstore->nzval == NULL);
-	}
-
-        /* ------------------------------------------------------------
-           Find the row permutation for A.
-           ------------------------------------------------------------*/
-        if ( options->RowPerm != NO ) {
-	    t = SuperLU_timer_();
-	    if ( Fact != SamePattern_SameRowPerm ) {
-	        if ( options->RowPerm == MY_PERMR ) { /* Use user's perm_r. */
-	            /* Permute the global matrix GA for symbfact() */
-	            for (i = 0; i < colptr[n]; ++i) {
-	            	irow = rowind[i]; 
-		    	rowind[i] = perm_r[irow];
-	            }
-	        } else { /* options->RowPerm == LargeDiag */
-	            /* Get a new perm_r[] */
-	            if ( job == 5 ) {
-		        /* Allocate storage for scaling factors. */
-		        if ( !(R1 = doubleMalloc_dist(m)) )
-		            ABORT("SUPERLU_MALLOC fails for R1[]");
-		    	if ( !(C1 = doubleMalloc_dist(n)) )
-		            ABORT("SUPERLU_MALLOC fails for C1[]");
-	            }
-
-	            if ( !iam ) {
-		        /* Process 0 finds a row permutation */
-		        dldperm(job, m, nnz, colptr, rowind, a_GA,
-		                perm_r, R1, C1);
+	    if ( !iam ) {
+		/* Process 0 finds a row permutation for large diagonal. */
+		dldperm(job, m, nnz, colptr, rowind, a_GA, perm_r, R1, C1);
 		
-		        MPI_Bcast( perm_r, m, mpi_int_t, 0, grid->comm );
-		        if ( job == 5 && Equil ) {
-		            MPI_Bcast( R1, m, MPI_DOUBLE, 0, grid->comm );
-		            MPI_Bcast( C1, n, MPI_DOUBLE, 0, grid->comm );
-		        }
-	            } else {
-		        MPI_Bcast( perm_r, m, mpi_int_t, 0, grid->comm );
-		        if ( job == 5 && Equil ) {
-		            MPI_Bcast( R1, m, MPI_DOUBLE, 0, grid->comm );
-		            MPI_Bcast( C1, n, MPI_DOUBLE, 0, grid->comm );
-		        }
-	            }
+		MPI_Bcast( perm_r, m, mpi_int_t, 0, grid->comm );
+		if ( job == 5 && Equil ) {
+		    MPI_Bcast( R1, m, MPI_DOUBLE, 0, grid->comm );
+		    MPI_Bcast( C1, n, MPI_DOUBLE, 0, grid->comm );
+		}
+	    } else {
+		MPI_Bcast( perm_r, m, mpi_int_t, 0, grid->comm );
+		if ( job == 5 && Equil ) {
+		    MPI_Bcast( R1, m, MPI_DOUBLE, 0, grid->comm );
+		    MPI_Bcast( C1, n, MPI_DOUBLE, 0, grid->comm );
+		}
+	    }
 
 #if ( PRNTlevel>=2 )
-	            dmin = dlamch_("Overflow");
-	            dsum = 0.0;
-	            dprod = 1.0;
+	    dmin = dlamch_("Overflow");
+	    dsum = 0.0;
+	    dprod = 1.0;
 #endif
-	            if ( job == 5 ) {
-		        if ( Equil ) {
-		            for (i = 0; i < n; ++i) {
-			        R1[i] = exp(R1[i]);
-			        C1[i] = exp(C1[i]);
-		            }
+	    if ( job == 5 ) {
+		if ( Equil ) {
+		    for (i = 0; i < n; ++i) {
+			R1[i] = exp(R1[i]);
+			C1[i] = exp(C1[i]);
+		    }
 
-		            /* Scale the distributed matrix */
-		            irow = fst_row;
-		            for (j = 0; j < m_loc; ++j) {
-			        for (i = rowptr[j]; i < rowptr[j+1]; ++i) {
-			            icol = colind[i];
-			            a[i] *= R1[irow] * C1[icol];
+		    /* Permute the global matrix GA for symbfact(). */
+		    for (j = 0; j < n; ++j) {
+			for (i = colptr[j]; i < colptr[j+1]; ++i) {
+			    irow = rowind[i];
+			    rowind[i] = perm_r[irow];
 #if ( PRNTlevel>=2 )
-			            if ( perm_r[irow] == icol ) { /* New diagonal */
-			              if ( job == 2 || job == 3 )
-				        dmin = SUPERLU_MIN(dmin, fabs(a[i]));
-			              else if ( job == 4 )
-				        dsum += fabs(a[i]);
-			              else if ( job == 5 )
-				        dprod *= fabs(a[i]);
-			            }
+			    if ( rowind[i] == j ) /* New diagonal */
+				dprod *= fabs(a[i]);
 #endif
-			        }
-			        ++irow;
-		            }
+			}
+		    }
 
-		            /* Multiply together the scaling factors. */
-		            if ( rowequ ) for (i = 0; i < m; ++i) R[i] *= R1[i];
-		            else for (i = 0; i < m; ++i) R[i] = R1[i];
-		            if ( colequ ) for (i = 0; i < n; ++i) C[i] *= C1[i];
-		            else for (i = 0; i < n; ++i) C[i] = C1[i];
+		    /* Scale the distributed matrix */
+		    irow = fst_row;
+		    for (j = 0; j < m_loc; ++j) {
+			for (i = rowptr[j]; i < rowptr[j+1]; ++i) {
+			  icol = colind[i];
+			  a[i] *= R1[irow] * C1[icol];
+			}
+			++irow;
+		    }
+
+		    /* Multiply together the scaling factors. */
+		    if ( rowequ ) for (i = 0; i < m; ++i) R[i] *= R1[i];
+		    else for (i = 0; i < m; ++i) R[i] = R1[i];
+		    if ( colequ ) for (i = 0; i < n; ++i) C[i] *= C1[i];
+		    else for (i = 0; i < n; ++i) C[i] = C1[i];
 		    
-		            ScalePermstruct->DiagScale = BOTH;
-		            rowequ = colequ = 1;
+		    ScalePermstruct->DiagScale = BOTH;
+		    rowequ = colequ = 1;
 
-		        } /* end Equil */
-
-                        /* Now permute global A to prepare for symbfact() */
-                        for (j = 0; j < n; ++j) {
-		            for (i = colptr[j]; i < colptr[j+1]; ++i) {
-	                        irow = rowind[i];
-		                rowind[i] = perm_r[irow];
-		            }
-		        }
-		        SUPERLU_FREE (R1);
-		        SUPERLU_FREE (C1);
-	            } else { /* job = 2,3,4 */
-		        for (j = 0; j < n; ++j) {
-		            for (i = colptr[j]; i < colptr[j+1]; ++i) {
-			        irow = rowind[i];
-			        rowind[i] = perm_r[irow];
-		            } /* end for i ... */
-		        } /* end for j ... */
-	            } /* end else job ... */
+		} else { /* No equilibration. Only permute the global A. */
+		    for (i = colptr[0]; i < colptr[n]; ++i) {
+		        irow = rowind[i];
+			rowind[i] = perm_r[irow];
+		    }
+		}
+		SUPERLU_FREE (R1);
+		SUPERLU_FREE (C1);
+	    } else { /* job = 2,3,4 */
+		for (j = 0; j < n; ++j) {
+		    for (i = colptr[j]; i < colptr[j+1]; ++i) {
+			irow = rowind[i];
+			rowind[i] = perm_r[irow];
+#if ( PRNTlevel>=2 )
+			if ( rowind[i] == j ) { /* New diagonal */
+			    if ( job == 2 || job == 3 )
+				dmin = SUPERLU_MIN(dmin, fabs(a[i]));
+			    else if ( job == 4 )
+				dsum += fabs(a[i]);
+			    else if ( job == 5 )
+				dprod *= fabs(a[i]);
+			}
+#endif
+		    }
+		}
+	    }
 
 #if ( PRNTlevel>=2 )
-	            if ( job == 2 || job == 3 ) {
-		        if ( !iam ) printf("\tsmallest diagonal %e\n", dmin);
-	            } else if ( job == 4 ) {
-		        if ( !iam ) printf("\tsum of diagonal %e\n", dsum);
-	            } else if ( job == 5 ) {
-		        if ( !iam ) printf("\t product of diagonal %e\n", dprod);
-	            }
+	    if ( job == 2 || job == 3 ) {
+		if ( !iam ) printf("\tsmallest diagonal %e\n", dmin);
+	    } else if ( job == 4 ) {
+		if ( !iam ) printf("\tsum of diagonal %e\n", dsum);
+	    } else if ( job == 5 ) {
+		if ( !iam ) printf("\t product of diagonal %e\n", dprod);
+	    }
 #endif
 	    
-                } /* end if options->RowPerm ... */
+        } /* else !factored */
 
-	        t = SuperLU_timer_() - t;
-	        stat->utime[ROWPERM] = t;
+	t = SuperLU_timer_() - t;
+	stat->utime[ROWPERM] = t;
 #if ( PRNTlevel>=1 )
-	        if ( !iam ) printf(".. LDPERM job %d\t time: %.2f\n", job, t);
+	if ( !iam ) printf(".. LDPERM job %d\t time: %.2f\n", job, t);
 #endif
-            } /* end if Fact ... */
-        } else { /* options->RowPerm == NOROWPERM */
-            for (i = 0; i < m; ++i) perm_r[i] = i;
-        }
+    
+    } else { /* options->RowPerm == NOROWPERM */
+        for (i = 0; i <m; ++i) perm_r[i] = i;
+    }
 
-#if ( DEBUGlevel>=2 )
-        if ( !iam ) PrintInt10("perm_r",  m, perm_r);
+#if ( DEBUGlevel>=1 )
+    if ( !iam ) PrintInt10("perm_r",  m, perm_r);
 #endif
-    } /* end if (!factored) */
 
     if ( !factored || options->IterRefine ) {
 	/* Compute norm(A), which will be used to adjust small diagonal. */
@@ -878,196 +834,113 @@ pdgssvx(superlu_options_t *options, SuperMatrix *A,
 	 *   permc_spec = NATURAL:  natural ordering 
 	 *   permc_spec = MMD_AT_PLUS_A: minimum degree on structure of A'+A
 	 *   permc_spec = MMD_ATA:  minimum degree on structure of A'*A
-	 *   permc_spec = METIS_AT_PLUS_A: METIS on structure of A'+A
-	 *   permc_spec = PARMETIS: parallel METIS on structure of A'+A
+	 *   permc_spec = COLAMD:   approximate minimum degree column ordering
 	 *   permc_spec = MY_PERMC: the ordering already supplied in perm_c[]
 	 */
 	permc_spec = options->ColPerm;
-
-
-	if ( parSymbFact == YES || permc_spec == PARMETIS ) {	
-	    nprocs_num = grid->nprow * grid->npcol;
-  	    noDomains = (int) ( pow(2, ((int) LOG2( nprocs_num ))));
-
-	    /* create a new communicator for the first noDomains processors in
-	       grid->comm */
-	    key = iam;
-    	    if (iam < noDomains) col = 0;
-	    else col = MPI_UNDEFINED;
-	    MPI_Comm_split (grid->comm, col, key, &symb_comm );
-
-	    permc_spec = PARMETIS; /* only works with PARMETIS */
-        }
-
-	if ( permc_spec != MY_PERMC && Fact == DOFACT ) {
-	  if ( permc_spec == PARMETIS ) {
-	      /* Get column permutation vector in perm_c.                    *
-	       * This routine takes as input the distributed input matrix A  *
-	       * and does not modify it.  It also allocates memory for       *
-	       * sizes[] and fstVtxSep[] arrays, that contain information    *
-	       * on the separator tree computed by ParMETIS.                 */
-	      flinfo = get_perm_c_parmetis(A, perm_r, perm_c, nprocs_num,
-                                  	   noDomains, &sizes, &fstVtxSep,
-                                           grid, &symb_comm);
-	      if (flinfo > 0)
-	          ABORT("ERROR in get perm_c parmetis.");
-	  } else {
-	      get_perm_c_dist(iam, permc_spec, &GA, perm_c);
-          }
-        }
-
-	stat->utime[COLPERM] = SuperLU_timer_() - t;
+	if ( permc_spec != MY_PERMC && Fact == DOFACT )
+	    get_perm_c_dist(iam, permc_spec, &GA, perm_c);
 
 	/* Compute the elimination tree of Pc*(A'+A)*Pc' or Pc*A'*A*Pc'
 	   (a.k.a. column etree), depending on the choice of ColPerm.
 	   Adjust perm_c[] to be consistent with a postorder of etree.
 	   Permute columns of A to form A*Pc'. */
-	if ( Fact != SamePattern_SameRowPerm ) {
-	    if ( parSymbFact == NO ) {
-	        int_t *GACcolbeg, *GACcolend, *GACrowind;
+	sp_colorder(options, &GA, perm_c, etree, &GAC); 
 
-	        sp_colorder(options, &GA, perm_c, etree, &GAC); 
-
-	        /* Form Pc*A*Pc' to preserve the diagonal of the matrix GAC. */
-	        GACstore = (NCPformat *) GAC.Store;
-	        GACcolbeg = GACstore->colbeg;
-	        GACcolend = GACstore->colend;
-	        GACrowind = GACstore->rowind;
-	        for (j = 0; j < n; ++j) {
-	            for (i = GACcolbeg[j]; i < GACcolend[j]; ++i) {
-		        irow = GACrowind[i];
-		        GACrowind[i] = perm_c[irow];
-	            }
-	        }
-
-	        /* Perform a symbolic factorization on Pc*Pr*A*Pc' and set up
-                   the nonzero data structures for L & U. */
-#if ( PRNTlevel>=1 ) 
-                if ( !iam )
-		  printf(".. symbfact(): relax %4d, maxsuper %4d, fill %4d\n",
-		          sp_ienv_dist(2), sp_ienv_dist(3), sp_ienv_dist(6));
-#endif
-  	        t = SuperLU_timer_();
-	        if ( !(Glu_freeable = (Glu_freeable_t *)
-		      SUPERLU_MALLOC(sizeof(Glu_freeable_t))) )
-		    ABORT("Malloc fails for Glu_freeable.");
-
-	    	/* Every process does this. */
-	    	iinfo = symbfact(options, iam, &GAC, perm_c, etree, 
-			     	 Glu_persist, Glu_freeable);
-
-	    	stat->utime[SYMBFAC] = SuperLU_timer_() - t;
-	    	if ( iinfo < 0 ) { /* Successful return */
-		    QuerySpace_dist(n, -iinfo, Glu_freeable, &symb_mem_usage);
-#if ( PRNTlevel>=1 )
-		    if ( !iam ) {
-		    	printf("\tNo of supers %ld\n", Glu_persist->supno[n-1]+1);
-		    	printf("\tSize of G(L) %ld\n", Glu_freeable->xlsub[n]);
-		    	printf("\tSize of G(U) %ld\n", Glu_freeable->xusub[n]);
-		    	printf("\tint %d, short %d, float %d, double %d\n", 
-			       sizeof(int_t), sizeof(short), sizeof(float),
-			       sizeof(double));
-		    	printf("\tSYMBfact (MB):\tL\\U %.2f\ttotal %.2f\texpansions %d\n",
-			   	symb_mem_usage.for_lu*1e-6, 
-			   	symb_mem_usage.total*1e-6,
-			   	symb_mem_usage.expansions);
-		    }
-#endif
-	    	} else {
-		    if ( !iam ) {
-		        fprintf(stderr,"symbfact() error returns %d\n",iinfo);
-		    	exit(-1);
-		    }
-	        }
-	    } /* end serial symbolic factorization */
-	    else {  /* parallel symbolic factorization */
-	    	t = SuperLU_timer_();
-	    	flinfo = symbfact_dist(nprocs_num, noDomains, A, perm_c, perm_r,
-				       sizes, fstVtxSep, &Pslu_freeable, 
-				       &(grid->comm), &symb_comm,
-				       &symb_mem_usage); 
-	    	stat->utime[SYMBFAC] = SuperLU_timer_() - t;
-	    	if (flinfo > 0) 
-	      	    ABORT("Insufficient memory for parallel symbolic factorization.");
+	/* Form Pc*A*Pc' to preserve the diagonal of the matrix GAC. */
+	{ 
+	  int_t *GACcolbeg, *GACcolend, *GACrowind;
+	  GACstore = GAC.Store;
+	  GACcolbeg = GACstore->colbeg;
+	  GACcolend = GACstore->colend;
+	  GACrowind = GACstore->rowind;
+	  for (j = 0; j < n; ++j) {
+	    for (i = GACcolbeg[j]; i < GACcolend[j]; ++i) {
+		irow = GACrowind[i];
+		GACrowind[i] = perm_c[irow];
 	    }
-
-            /* Destroy GA */
-            if ( parSymbFact == NO || options->RowPerm != NO )
-                Destroy_CompCol_Matrix_dist(&GA);
-            if ( parSymbFact == NO )
- 	        Destroy_CompCol_Permuted_dist(&GAC);
-
-	} /* end if Fact ... */
-
-        if (sizes) SUPERLU_FREE (sizes);
-        if (fstVtxSep) SUPERLU_FREE (fstVtxSep);
-	if (symb_comm != MPI_COMM_NULL)
-	  MPI_Comm_free (&symb_comm); 
-
-	if (parSymbFact == NO || Fact == SamePattern_SameRowPerm) {
-  	    /* Apply column permutation to the original distributed A */
-	    for (j = 0; j < nnz_loc; ++j) colind[j] = perm_c[colind[j]];
-
-	    /* Distribute Pc*Pr*diag(R)*A*diag(C)*Pc' into L and U storage. 
-	       NOTE: the row permutation Pc*Pr is applied internally in the
-  	       distribution routine. */
-	    t = SuperLU_timer_();
-	    dist_mem_use = pddistribute(Fact, n, A, ScalePermstruct,
-                                      Glu_freeable, LUstruct, grid);
-	    stat->utime[DIST] = SuperLU_timer_() - t;
-
-  	    /* Deallocate storage used in symbolic factorization. */
-	    if ( Fact != SamePattern_SameRowPerm ) {
-	        iinfo = symbfact_SubFree(Glu_freeable);
-	        SUPERLU_FREE(Glu_freeable);
-	    }
-	} else {
-	    /* Distribute Pc*Pr*diag(R)*A*diag(C)*Pc' into L and U storage. 
-	       NOTE: the row permutation Pc*Pr is applied internally in the
-	       distribution routine. */
-	    /* Apply column permutation to the original distributed A */
-	    for (j = 0; j < nnz_loc; ++j) colind[j] = perm_c[colind[j]];
-
-    	    t = SuperLU_timer_();
-	    dist_mem_use = ddist_psymbtonum(Fact, n, A, ScalePermstruct,
-		  			   &Pslu_freeable, LUstruct, grid);
-	    if (dist_mem_use > 0)
-	        ABORT ("Not enough memory available for dist_psymbtonum\n");
-            
-	    stat->utime[DIST] = SuperLU_timer_() - t;
+	  }
 	}
 
-	/*if (!iam) printf ("\tDISTRIBUTE time  %8.2f\n", stat->utime[DIST]);*/
+	stat->utime[COLPERM] = SuperLU_timer_() - t;
+
+	/* Perform a symbolic factorization on Pc*Pr*A*Pc' and set up the
+	   nonzero data structures which are suitable for supernodal GENP. */
+	if ( Fact != SamePattern_SameRowPerm ) {
+#if ( PRNTlevel>=1 ) 
+	    if ( !iam ) 
+		printf(".. symbfact(): relax %4d, maxsuper %4d, fill %4d\n",
+		       sp_ienv_dist(2), sp_ienv_dist(3), sp_ienv_dist(6));
+#endif
+	    t = SuperLU_timer_();
+	    if ( !(Glu_freeable = (Glu_freeable_t *)
+		   SUPERLU_MALLOC(sizeof(Glu_freeable_t))) )
+		ABORT("Malloc fails for Glu_freeable.");
+
+	    /* Every process does this. */
+	    iinfo = symbfact(iam, &GAC, perm_c, etree, 
+			     Glu_persist, Glu_freeable);
+
+	    stat->utime[SYMBFAC] = SuperLU_timer_() - t;
+	    if ( iinfo < 0 ) { /* Successful return */
+		QuerySpace_dist(n, -iinfo, Glu_freeable, &symb_mem_usage);
+#if ( PRNTlevel>=1 )
+		if ( !iam ) {
+		    printf("\tNo of supers %ld\n", Glu_persist->supno[n-1]+1);
+		    printf("\tSize of G(L) %ld\n", Glu_freeable->xlsub[n]);
+		    printf("\tSize of G(U) %ld\n", Glu_freeable->xusub[n]);
+		    printf("\tint %d, short %d, float %d, double %d\n", 
+			   sizeof(int_t), sizeof(short), sizeof(float),
+			   sizeof(double));
+		    printf("\tSYMBfact (MB):\tL\\U %.2f\ttotal %.2f\texpansions %d\n",
+			   symb_mem_usage.for_lu*1e-6, 
+			   symb_mem_usage.total*1e-6,
+			   symb_mem_usage.expansions);
+		}
+#endif
+	    } else {
+		if ( !iam ) {
+		    fprintf(stderr, "symbfact() error returns %d\n", iinfo);
+		    exit(-1);
+		}
+	    }
+	}
+
+	/* Apply column permutation to the original distributed A */
+	for (j = 0; j < nnz_loc; ++j) colind[j] = perm_c[colind[j]];
+
+	/* Distribute Pc*Pr*diag(R)*A*diag(C)*Pc' into L and U storage. 
+	   NOTE: the row permutation Pc*Pr is applied internally in the
+	   distribution routine. */
+	t = SuperLU_timer_();
+	dist_mem_use = pddistribute(Fact, n, A, ScalePermstruct,
+                                  Glu_freeable, LUstruct, grid);
+	stat->utime[DIST] = SuperLU_timer_() - t;
+
+	/* Deallocate storage used in symbolic factorization. */
+	if ( Fact != SamePattern_SameRowPerm ) {
+	    iinfo = symbfact_SubFree(Glu_freeable);
+	    SUPERLU_FREE(Glu_freeable);
+	}
 
 	/* Perform numerical factorization in parallel. */
 	t = SuperLU_timer_();
 	pdgstrf(options, m, n, anorm, LUstruct, grid, stat, info);
 	stat->utime[FACT] = SuperLU_timer_() - t;
 
-	if ( options->PrintStat ) {
+#if ( PRNTlevel>=1 )
+	{
 	    int_t TinyPivots;
 	    float for_lu, total, max, avg, temp;
-
 	    dQuerySpace_dist(n, LUstruct, grid, &num_mem_usage);
-
-	    if (parSymbFact == TRUE) {
-	        /* The memory used in the redistribution routine
-		   includes the memory used for storing the symbolic
-  		   structure and the memory allocated for numerical
-		   factorization */
-	        temp = SUPERLU_MAX(symb_mem_usage.total, -dist_mem_use);
-                if ( options->RowPerm != NO )
-                    temp = SUPERLU_MAX(temp, GA_mem_use);
-            } else {
-	        temp = SUPERLU_MAX (
-                         symb_mem_usage.total + GA_mem_use, /* symbfact step */
-		         symb_mem_usage.for_lu + dist_mem_use +
-                             num_mem_usage.for_lu  /* distribution step */
-                       );
-            }
-            
+	    MPI_Reduce( &num_mem_usage.for_lu, &for_lu,
+		       1, MPI_FLOAT, MPI_SUM, 0, grid->comm );
+	    MPI_Reduce( &num_mem_usage.total, &total,
+		       1, MPI_FLOAT, MPI_SUM, 0, grid->comm );
+	    temp = SUPERLU_MAX(symb_mem_usage.total,
+			       symb_mem_usage.for_lu +
+			       (float)dist_mem_use + num_mem_usage.for_lu);
 	    temp = SUPERLU_MAX(temp, num_mem_usage.total);
-
 	    MPI_Reduce( &temp, &max,
 		       1, MPI_FLOAT, MPI_MAX, 0, grid->comm );
 	    MPI_Reduce( &temp, &avg,
@@ -1075,23 +948,32 @@ pdgssvx(superlu_options_t *options, SuperMatrix *A,
 	    MPI_Allreduce( &stat->TinyPivots, &TinyPivots, 1, mpi_int_t,
 			  MPI_SUM, grid->comm );
 	    stat->TinyPivots = TinyPivots;
-
-	    MPI_Reduce( &num_mem_usage.for_lu, &for_lu,
-		       1, MPI_FLOAT, MPI_SUM, 0, grid->comm );
-	    MPI_Reduce( &num_mem_usage.total, &total,
-		       1, MPI_FLOAT, MPI_SUM, 0, grid->comm );
-
 	    if ( !iam ) {
-		printf("\tNUMfact space (MB) sum(procs):  L\\U\t%.2f\tall\t%.2f\n",
+		printf("\tNUMfact (MB) all PEs:\tL\\U\t%.2f\tall\t%.2f\n",
 		       for_lu*1e-6, total*1e-6);
-		printf("\tTotal highmark (MB):  "
-		       "All\t%.2f\tAvg\t%.2f\tMax\t%.2f\n",
+		printf("\tAll space (MB):"
+		       "\t\ttotal\t%.2f\tAvg\t%.2f\tMax\t%.2f\n",
 		       avg*1e-6, avg/grid->nprow/grid->npcol*1e-6, max*1e-6);
+		printf("\tNumber of tiny pivots: %10d\n", stat->TinyPivots);
 	    }
 	}
+#endif
     
-    } /* end if (!factored) */
+    } else if ( options->IterRefine ) { /* options->Fact==FACTORED */
+	/* Permute columns of A to form A*Pc' using the existing perm_c.
+	 * NOTE: rows of A were previously permuted to Pc*A.
+	 *
+	 * XSL: NO; this is different now.
+	 */
+	sp_colorder(options, &GA, perm_c, NULL, &GAC); /* ????? */
+    } /* if !factored ... */
+
+    /* Destroy GA */ 
+    Destroy_CompCol_Matrix_dist(&GA);
 	
+#if ( DEBUGlevel>=1 )
+    CHECK_MALLOC(iam, "Before solve");
+#endif
     /* ------------------------------------------------------------
        Compute the solution matrix X.
        ------------------------------------------------------------*/
@@ -1148,90 +1030,20 @@ pdgssvx(superlu_options_t *options, SuperMatrix *A,
 	pdgstrs(n, LUstruct, ScalePermstruct, grid, X, m_loc, 
 		fst_row, ldb, nrhs, SOLVEstruct, stat, info);
 
+#if ( DEBUGlevel>=2 )
+	printf("\n(%d) .. After pdgstrs(): x =\n", iam);
+	for (i = 0; i < m_loc; ++i)
+	  printf("\t(%d)\t%4d\t%.10f\n", iam, i+fst_row, X[i]);
+#endif
 	/* ------------------------------------------------------------
 	   Use iterative refinement to improve the computed solution and
 	   compute error bounds and backward error estimates for it.
 	   ------------------------------------------------------------*/
 	if ( options->IterRefine ) {
 	    /* Improve the solution by iterative refinement. */
-	    int_t *it, *colind_gsmv = SOLVEstruct->A_colind_gsmv;
-	    SOLVEstruct_t *SOLVEstruct1;  /* Used by refinement. */
-
 	    t = SuperLU_timer_();
-	    if ( options->RefineInitialized == NO || Fact == DOFACT ) {
-	        /* All these cases need to re-initialize gsmv structure */
-	        if ( options->RefineInitialized )
-		    pdgsmv_finalize(SOLVEstruct->gsmv_comm);
-	        pdgsmv_init(A, SOLVEstruct->row_to_proc, grid,
-			    SOLVEstruct->gsmv_comm);
-	       
-                /* Save a copy of the transformed local col indices
-		   in colind_gsmv[]. */
-	        if ( colind_gsmv ) SUPERLU_FREE(colind_gsmv);
-	        if ( !(it = intMalloc_dist(nnz_loc)) )
-		    ABORT("Malloc fails for colind_gsmv[]");
-	        colind_gsmv = SOLVEstruct->A_colind_gsmv = it;
-	        for (i = 0; i < nnz_loc; ++i) colind_gsmv[i] = colind[i];
-	        options->RefineInitialized = YES;
-	    } else if ( Fact == SamePattern ||
-			Fact == SamePattern_SameRowPerm ) {
-	        double at;
-	        int_t k, jcol, p;
-	        /* Swap to beginning the part of A corresponding to the
-		   local part of X, as was done in pdgsmv_init() */
-	        for (i = 0; i < m_loc; ++i) { /* Loop through each row */
-		    k = rowptr[i];
-		    for (j = rowptr[i]; j < rowptr[i+1]; ++j) {
-		        jcol = colind[j];
-		        p = SOLVEstruct->row_to_proc[jcol];
-		        if ( p == iam ) { /* Local */
-		            at = a[k]; a[k] = a[j]; a[j] = at;
-		            ++k;
-		        }
-		    }
-	        }
-	      
-	        /* Re-use the local col indices of A obtained from the
-		   previous call to pdgsmv_init() */
-	        for (i = 0; i < nnz_loc; ++i) colind[i] = colind_gsmv[i];
-	    }
-
-	    if ( nrhs == 1 ) { /* Use the existing solve structure */
-	        SOLVEstruct1 = SOLVEstruct;
-	    } else { /* For nrhs > 1, since refinement is performed for RHS
-			one at a time, the communication structure for pdgstrs
-			is different than the solve with nrhs RHS. 
-			So we use SOLVEstruct1 for the refinement step.
-		     */
-	        if ( !(SOLVEstruct1 = (SOLVEstruct_t *) 
-		                       SUPERLU_MALLOC(sizeof(SOLVEstruct_t))) )
-		    ABORT("Malloc fails for SOLVEstruct1");
-	        /* Copy the same stuff */
-	        SOLVEstruct1->row_to_proc = SOLVEstruct->row_to_proc;
-	        SOLVEstruct1->inv_perm_c = SOLVEstruct->inv_perm_c;
-	        SOLVEstruct1->num_diag_procs = SOLVEstruct->num_diag_procs;
-	        SOLVEstruct1->diag_procs = SOLVEstruct->diag_procs;
-	        SOLVEstruct1->diag_len = SOLVEstruct->diag_len;
-	        SOLVEstruct1->gsmv_comm = SOLVEstruct->gsmv_comm;
-	        SOLVEstruct1->A_colind_gsmv = SOLVEstruct->A_colind_gsmv;
-		
-		/* Initialize the *gstrs_comm for 1 RHS. */
-		if ( !(SOLVEstruct1->gstrs_comm = (pxgstrs_comm_t *)
-		       SUPERLU_MALLOC(sizeof(pxgstrs_comm_t))) )
-		    ABORT("Malloc fails for gstrs_comm[]");
-		pxgstrs_init(n, m_loc, 1, fst_row, perm_r, perm_c, grid, 
-			     Glu_persist, SOLVEstruct1);
-	    }
-
 	    pdgsrfs(n, A, anorm, LUstruct, ScalePermstruct, grid,
-		    B, ldb, X, ldx, nrhs, SOLVEstruct1, berr, stat, info);
-
-            /* Deallocate the storage associated with SOLVEstruct1 */
-	    if ( nrhs > 1 ) {
-	        pxgstrs_finalize(SOLVEstruct1->gstrs_comm);
-	        SUPERLU_FREE(SOLVEstruct1);
-	    }
-
+		    B, ldb, X, ldx, nrhs, SOLVEstruct, berr, stat, info);
 	    stat->utime[REFINE] = SuperLU_timer_() - t;
 	}
 
@@ -1280,7 +1092,7 @@ pdgssvx(superlu_options_t *options, SuperMatrix *A,
     if ( !iam ) printf(".. DiagScale = %d\n", ScalePermstruct->DiagScale);
 #endif
 
-    /* Deallocate R and/or C if it was not used. */
+    /* Deallocate storage. */
     if ( Equil && Fact != SamePattern_SameRowPerm ) {
 	switch ( ScalePermstruct->DiagScale ) {
 	    case NOEQUIL:
@@ -1295,11 +1107,9 @@ pdgssvx(superlu_options_t *options, SuperMatrix *A,
 		break;
 	}
     }
+    if ( !factored || (factored && options->IterRefine) )
+	Destroy_CompCol_Permuted_dist(&GAC);
 
-#if 0
-    if ( !factored && Fact != SamePattern_SameRowPerm && !parSymbFact)
- 	Destroy_CompCol_Permuted_dist(&GAC);
-#endif
 #if ( DEBUGlevel>=1 )
     CHECK_MALLOC(iam, "Exit pdgssvx()");
 #endif
