@@ -14,7 +14,6 @@
 /* limits.h:  the largest positive integer (INT_MAX) */
 #include <limits.h>
 #include <math.h>
-#include "parmetis.h"
 #include "superlu_ddefs.h"
 
 /*
@@ -96,13 +95,9 @@ get_perm_c_parmetis (SuperMatrix *A, int_t *perm_r, int_t *perm_c,
 {
   NRformat_loc *Astore;
   int   iam, p;
-#if 0
   int   *b_rowptr_int, *b_colind_int, *l_sizes_int, *dist_order_int, *vtxdist_o_int;
   int   *options, numflag;
-#else /* 64-bit integers */
-  int_t options[4]={0,0,0,1}, numflag;
-#endif
-  int_t m_loc, fst_row;
+  int_t m_loc, nnz_loc, fst_row;
   int_t m, n, bnz, i, j;
   int_t *rowptr, *colind, *l_fstVtxSep, *l_sizes;
   int_t *b_rowptr, *b_colind;
@@ -128,6 +123,7 @@ get_perm_c_parmetis (SuperMatrix *A, int_t *perm_r, int_t *perm_c,
 #endif
 
   Astore = (NRformat_loc *) A->Store;
+  nnz_loc = Astore->nnz_loc; /* number of nonzeros in the local submatrix */
   m_loc = Astore->m_loc;     /* number of rows local to this processor */
   fst_row = Astore->fst_row; /* global index of the first row */
   rowptr = Astore->rowptr;   /* pointer to rows and column indices */
@@ -191,11 +187,9 @@ get_perm_c_parmetis (SuperMatrix *A, int_t *perm_r, int_t *perm_c,
   m_loc = vtxdist_o[iam+1] - vtxdist_o[iam];
   
   if ( iam < noDomains) 
-    /* dist_order is the perm returned by parMetis, distributed */
-    if (! (dist_order = (int_t *) SUPERLU_MALLOC(m_loc * sizeof(int_t))))
-      ABORT("SUPERLU_MALLOC fails for dist_order.");
-
-#if 0  /* Obsolate -- now ParMETIS has 64 bit integer support. */
+    /* dist_order_int is the perm returned by parMetis, distributed */
+    if (! (dist_order_int = (int *) SUPERLU_MALLOC(m_loc * sizeof(int))))
+      ABORT("SUPERLU_MALLOC fails for dist_order_int.");
 
   /* ParMETIS represents the column pointers and row indices of *
    * the input matrix using integers. When SuperLU_DIST uses    *
@@ -233,24 +227,27 @@ get_perm_c_parmetis (SuperMatrix *A, int_t *perm_r, int_t *perm_c,
   b_rowptr_int = b_rowptr; b_colind_int = b_colind;
   l_sizes_int = l_sizes;
 
-#endif
 #endif
     
   if ( iam < noDomains) {
+    options = (int *) SUPERLU_MALLOC(4 * sizeof(int));
+    options[0] = 0;
+    options[1] = 0;
+    options[2] = 0;
+    options[3] = 1;
 
-    ParMETIS_V3_NodeND(vtxdist_o, b_rowptr, b_colind, 
+    ParMETIS_V3_NodeND(vtxdist_o_int, b_rowptr_int, b_colind_int, 
 		       &numflag, options,
-		       dist_order, l_sizes, metis_comm);
+		       dist_order_int, l_sizes_int, metis_comm);
   }
-
-  if (bnz) SUPERLU_FREE (b_colind);
-  SUPERLU_FREE (b_rowptr);
-
-#if 0  
+  
+  if (bnz) 
+    SUPERLU_FREE (b_colind_int);
   if ( iam < noDomains) {
     SUPERLU_FREE (options);
   }
-
+  SUPERLU_FREE (b_rowptr_int);
+  
 #if defined (_LONGINT)
   /* Copy data from dist_order_int to dist_order */
   if ( iam < noDomains) {
@@ -267,8 +264,6 @@ get_perm_c_parmetis (SuperMatrix *A, int_t *perm_r, int_t *perm_c,
   }
 #else 
   dist_order = dist_order_int;
-#endif
-
 #endif
   
   /* Allgatherv dist_order to get perm_c */
@@ -277,7 +272,7 @@ get_perm_c_parmetis (SuperMatrix *A, int_t *perm_r, int_t *perm_c,
   if ( !(recvcnts = (int *) SUPERLU_MALLOC (nprocs_i * sizeof(int))))
     ABORT ("SUPERLU_MALLOC fails for recvcnts.");
   for (i = 0; i < nprocs_i; i++)
-    recvcnts[i] = vtxdist_o[i+1] - vtxdist_o[i];
+    recvcnts[i] = vtxdist_o_int[i+1] - vtxdist_o_int[i];
   displs[0]=0;
   for(i=1; i < nprocs_i; i++) 
     displs[i] = displs[i-1] + recvcnts[i-1];
@@ -289,7 +284,7 @@ get_perm_c_parmetis (SuperMatrix *A, int_t *perm_r, int_t *perm_c,
     SUPERLU_FREE (dist_order);
   }
   SUPERLU_FREE (vtxdist_i);
-  SUPERLU_FREE (vtxdist_o);
+  SUPERLU_FREE (vtxdist_o_int);
   SUPERLU_FREE (recvcnts);
   SUPERLU_FREE (displs);
   
@@ -414,13 +409,13 @@ a_plus_at_CompRow_loc
   int_t i, j, k, col, num_nz, nprocs;
   int_t *tcolind_recv; /* temporary receive buffer */
   int_t *tcolind_send; /* temporary send buffer */
-  int_t sz_tcolind_send, sz_tcolind_recv;
-  int_t ind, ind_rcv;
+  int_t sz_tcolind_send, sz_tcolind_loc, sz_tcolind_recv;
+  int_t ind, ind_tmp, ind_rcv;
   int redist_pra; /* TRUE if Pr != I or nprocs_i != nprocs_o */
   int_t *marker, *iperm_r;
   int_t *sendCnts, *recvCnts;
   int_t *sdispls, *rdispls;
-  int_t *b_rowptr, *b_colind, bnz_t, *b_rowptr_t, *b_colind_t;
+  int_t bnz, *b_rowptr, *b_colind, bnz_t, *b_rowptr_t, *b_colind_t;
   int_t p, t_ind, nelts, ipcol;
   int_t m_loc, m_loc_o;      /* number of local rows */ 
   int_t fst_row, fst_row_o;  /* index of first local row */
@@ -489,6 +484,7 @@ a_plus_at_CompRow_loc
   MPI_Alltoall (sendCnts, 1, mpi_int_t,
 		recvCnts, 1, mpi_int_t, grid->comm);
   sendCnts[iam] = 0;
+  sz_tcolind_loc = recvCnts[iam];
   
   for (i = 0, j = 0, p = 0; p < nprocs_i; p++) {
     rdispls[p] = j;
@@ -539,7 +535,7 @@ a_plus_at_CompRow_loc
     sdispls[p] = i;  
     i += sendCnts[p];
   }
-  /* Second pass of the local matrix A to copy data to be sent */
+  /* Second pass of the local matrix A to copy data to be send */
   for (j = 0; j < m_loc; j++)
     for (i = rowptr[j]; i < rowptr[j+1]; i++) {
       col = colind[i];
@@ -760,7 +756,7 @@ a_plus_at_CompRow_loc
     if (apat_mem > apat_mem_max)
       apat_mem_max = apat_mem;
 
-    /* Copy data to be sent */
+    /* Copy data to be send */
     ind_rcv = rdispls[iam];
     for (i = 0; i < m_loc; i++) {
       k = perm_r[i+fst_row];
@@ -833,14 +829,13 @@ a_plus_at_CompRow_loc
       }
     }
 
-    if (nnz_loc) {
+    if (nnz_loc)
       if ( !(b_colind = intMalloc_dist(nnz_loc)) ) {
 	ABORT("Malloc fails for bcolind[].");
 	apat_mem += nnz_loc;
 	if (apat_mem > apat_mem_max)
 	  apat_mem_max = apat_mem;
       }
-    }
 
     /* Initialize the array of row pointers */
     k = 0;
@@ -884,7 +879,7 @@ a_plus_at_CompRow_loc
     *p_b_rowptr = b_rowptr;
     *p_b_colind = b_colind;
   }
-  else { /* no need for redistribution */
+  else {
     *p_bnz = bnz_t;
     *p_b_rowptr = b_rowptr_t;
     *p_b_colind = b_colind_t;
