@@ -2,16 +2,16 @@
  * \brief Utilities functions
  *
  * <pre>
- * -- Distributed SuperLU routine (version 2.3) --
+ * -- Distributed SuperLU routine (version 2.0) --
  * Lawrence Berkeley National Lab, Univ. of California Berkeley.
  * February 1, 2003
- * Modified: March 31, 2013
  * </pre>
  */
 
 #include <math.h>
 #include "superlu_ddefs.h"
 
+
 /*! \brief Deallocate the structure pointing to the actual storage of the matrix. */
 void
 Destroy_SuperMatrix_Store_dist(SuperMatrix *A)
@@ -98,11 +98,7 @@ Destroy_LU(int_t n, gridinfo_t *grid, LUstruct_t *LUstruct)
     for (i = 0; i < nb; ++i) 
 	if ( Llu->Lrowind_bc_ptr[i] ) {
 	    SUPERLU_FREE (Llu->Lrowind_bc_ptr[i]);
-#ifdef GPU_ACC
-	    checkCuda(cudaFreeHost(Llu->Lnzval_bc_ptr[i]));
-#else
 	    SUPERLU_FREE (Llu->Lnzval_bc_ptr[i]);
-#endif
 	}
     SUPERLU_FREE (Llu->Lrowind_bc_ptr);
     SUPERLU_FREE (Llu->Lnzval_bc_ptr);
@@ -171,7 +167,7 @@ void ScalePermstructFree(ScalePermstruct_t *ScalePermstruct)
 }
 
 /*! \brief Allocate storage in LUstruct */
-void LUstructInit(const int_t n, LUstruct_t *LUstruct)
+void LUstructInit(const int_t m, const int_t n, LUstruct_t *LUstruct)
 {
     if ( !(LUstruct->etree = intMalloc_dist(n)) )
 	ABORT("Malloc fails for etree[].");
@@ -209,13 +205,11 @@ void LUstructFree(LUstruct_t *LUstruct)
  * </pre>
  */
 void
-countnz_dist(const int_t n, int_t *xprune,
-	     long long int *nnzL, long long int *nnzU, 
+countnz_dist(const int_t n, int_t *xprune, int_t *nnzL, int_t *nnzU, 
 	     Glu_persist_t *Glu_persist, Glu_freeable_t *Glu_freeable)
 {
     int_t  fnz, fsupc, i, j, nsuper;
-    int_t  jlen, irep;
-    long long int nnzL0;
+    int_t  nnzL0, jlen, irep;
     int_t  *supno, *xsup, *xlsub, *xusub, *usub;
 
     supno  = Glu_persist->supno;
@@ -246,7 +240,7 @@ countnz_dist(const int_t n, int_t *xprune,
 	nnzL0 += xprune[irep] - xlsub[irep];
     }
     
-    /* printf("\tNo of nonzeros in symm-reduced L = %ld\n", nnzL0);*/
+    /* printf("\tNo of nonzeros in symm-reduced L = %d\n", nnzL0);*/
     
     /* For each column in U. */
     for (j = 0; j < n; ++j) {
@@ -268,12 +262,11 @@ countnz_dist(const int_t n, int_t *xprune,
  * subscripts.
  * </pre>
  */
-long long int
+int_t
 fixupL_dist(const int_t n, const int_t *perm_r, 
 	    Glu_persist_t *Glu_persist, Glu_freeable_t *Glu_freeable)
 {
-    register int_t nsuper, fsupc, nextl, i, j, k, jstrt;
-    register long long int lsub_size;
+    register int_t nsuper, fsupc, nextl, i, j, k, jstrt, lsub_size;
     int_t          *xsup, *lsub, *xlsub;
 
     if ( n <= 1 ) return 0;
@@ -315,53 +308,28 @@ void set_default_options_dist(superlu_options_t *options)
     options->ColPerm           = METIS_AT_PLUS_A;
     options->RowPerm           = LargeDiag;
     options->ReplaceTinyPivot  = YES;
-    options->IterRefine        = SLU_DOUBLE;
+    options->IterRefine        = DOUBLE;
     options->Trans             = NOTRANS;
     options->SolveInitialized  = NO;
     options->RefineInitialized = NO;
     options->PrintStat         = YES;
-    options->num_lookaheads    = 10;
-    options->lookahead_etree   = NO;
-    options->SymPattern        = NO;
 }
 
 /*! \brief Print the options setting.
  */
 void print_options_dist(superlu_options_t *options)
 {
-    if ( options->PrintStat == NO ) return;
-
-    printf("**************************************************\n");
     printf(".. options:\n");
-    printf("**    Fact             : %4d\n", options->Fact);
-    printf("**    Equil            : %4d\n", options->Equil);
-    printf("**    ParSymbFact      : %4d\n", options->ParSymbFact);
-    printf("**    ColPerm          : %4d\n", options->ColPerm);
-    printf("**    RowPerm          : %4d\n", options->RowPerm);
-    printf("**    ReplaceTinyPivot : %4d\n", options->ReplaceTinyPivot);
-    printf("**    IterRefine       : %4d\n", options->IterRefine);
-    printf("**    Trans            : %4d\n", options->Trans);
-    printf("**    num_lookaheads   : %4d\n", options->num_lookaheads);
-    printf("**    SymPattern       : %4d\n", options->SymPattern);
-    printf("**    lookahead_etree  : %4d\n", options->lookahead_etree);
-    printf("**************************************************\n");
+    printf("\tFact\t %8d\n", options->Fact);
+    printf("\tEquil\t %8d\n", options->Equil);
+    printf("\tColPerm\t %8d\n", options->ColPerm);
+    printf("\tRowPerm\t %8d\n", options->RowPerm);
+    printf("\tReplaceTinyPivot %4d\n", options->ReplaceTinyPivot);
+    printf("\tTrans\t %8d\n", options->Trans);
+    printf("\tIterRefine\t%4d\n", options->IterRefine);
+    printf("..\n");
 }
 
-/*! \brief Print the blocking parameters.
- */
-void print_sp_ienv_dist(superlu_options_t *options)
-{
-    if ( options->PrintStat == NO ) return;
-
-    printf("**************************************************\n");
-    printf(".. blocking parameters from sp_ienv():\n");
-    printf("**    relaxation           : " IFMT "\n", sp_ienv_dist(2));
-    printf("**    max supernode        : " IFMT "\n", sp_ienv_dist(3));
-    printf("**    estimated fill ratio : " IFMT "\n", sp_ienv_dist(6));
-    printf("**************************************************\n");
-}
-
-
 /*! \brief
  *
  * <pre>
@@ -405,7 +373,7 @@ pxgstrs_init(int_t n, int_t m_loc, int_t nrhs, int_t fst_row,
     int *itemp, *ptr_to_ibuf, *ptr_to_dbuf;
     int_t *row_to_proc;
     int_t i, gbi, k, l, num_diag_procs, *diag_procs;
-    int_t irow, q, knsupc, nsupers, *xsup, *supno;
+    int_t irow, lk, q, knsupc, nsupers, *xsup, *supno;
     int   iam, p, pkk, procs;
     pxgstrs_comm_t *gstrs_comm;
 
@@ -482,6 +450,7 @@ pxgstrs_init(int_t n, int_t m_loc, int_t nrhs, int_t fst_row,
 	if ( iam == pkk ) {
 	    for (k = p; k < nsupers; k += num_diag_procs) {
 		knsupc = SuperSize( k );
+		lk = LBi( k, grid ); /* Local block number */
 		irow = FstBlockC( k );
 		for (i = 0; i < knsupc; ++i) {
 #if 0
@@ -518,7 +487,6 @@ pxgstrs_init(int_t n, int_t m_loc, int_t nrhs, int_t fst_row,
     gstrs_comm->ptr_to_ibuf = ptr_to_ibuf;
     gstrs_comm->ptr_to_dbuf = ptr_to_ibuf + procs;
 
-    return 0;
 } /* PXGSTRS_INIT */
 
 
@@ -539,9 +507,9 @@ void print_panel_seg_dist(int_t n, int_t w, int_t jcol, int_t nseg,
     int_t j, k;
     
     for (j = jcol; j < jcol+w; j++) {
-	printf("\tcol " IFMT ":\n", j);
+	printf("\tcol %d:\n", j);
 	for (k = 0; k < nseg; k++)
-	    printf("\t\tseg " IFMT ", segrep " IFMT ", repfnz " IFMT "\n", k, 
+	    printf("\t\tseg %d, segrep %d, repfnz %d\n", k, 
 			segrep[k], repfnz[(j-jcol)*n + segrep[k]]);
     }
 
@@ -574,9 +542,6 @@ PStatPrint(superlu_options_t *options, SuperLUStat_t *stat, gridinfo_t *grid)
     if ( options->PrintStat == NO ) return;
 
     if ( !iam && options->Fact != FACTORED ) {
-	printf("**************************************************\n");
-	printf("**** Time (seconds) ****\n");
-
         if ( options->Equil != NO )
 	    printf("\tEQUIL time         %8.2f\n", utime[EQUIL]);
 	if ( options->RowPerm != NOROWPERM )
@@ -608,11 +573,11 @@ PStatPrint(superlu_options_t *options, SuperLUStat_t *stat, gridinfo_t *grid)
 	    printf("\tSolve flops\t%e\tMflops \t%8.2f\n",
 		   flopcnt,
 		   flopcnt*1e-6/utime[SOLVE]);
-	if ( options->IterRefine != NOREFINE ) {
-	    printf("\tREFINEMENT time    %8.2f\tSteps%8d\n\n",
-		   utime[REFINE], stat->RefineSteps);
-	}
-	printf("**************************************************\n");
+    }
+    
+    if ( !iam && options->IterRefine != NOREFINE ) {
+	printf("\tREFINEMENT time    %8.2f\tSteps%8d\n\n",
+	       utime[REFINE], stat->RefineSteps);
     }
 
 #if ( PROFlevel>=1 )
@@ -734,9 +699,9 @@ void super_stats_dist(int_t nsuper, int_t *xsup)
 	if ( max_sup_size < isize ) max_sup_size = isize;	
     }
 
-    printf("    Supernode statistics:\n\tno of super = " IFMT "\n", nsuper+1);
-    printf("\tmax supernode size = " IFMT "\n", max_sup_size);
-    printf("\tno of size 1 supernodes = " IFMT "\n", nsup1);
+    printf("    Supernode statistics:\n\tno of super = %d\n", nsuper+1);
+    printf("\tmax supernode size = %d\n", max_sup_size);
+    printf("\tno of size 1 supernodes = %d\n", nsup1);
 
     /* Histogram of the supernode sizes */
     ifill_dist (bucket, NBUCKS, 0);
@@ -752,7 +717,7 @@ void super_stats_dist(int_t nsuper, int_t *xsup)
     for (i = 0; i < NBUCKS; i++) {
         bl = (float) i * max_sup_size / NBUCKS;
         bh = (float) (i+1) * max_sup_size / NBUCKS;
-        printf("\tsnode: " IFMT "-" IFMT "\t\t" IFMT "\n", bl+1, bh, bucket[i]);
+        printf("\tsnode: %d-%d\t\t%d\n", bl+1, bh, bucket[i]);
     }
 
 }
@@ -767,8 +732,8 @@ void check_repfnz_dist(int_t n, int_t w, int_t jcol, int_t *repfnz)
     for (jj = jcol; jj < jcol+w; jj++) 
 	for (k = 0; k < n; k++)
 	    if ( repfnz[(jj-jcol)*n + k] != EMPTY ) {
-		fprintf(stderr, "col " IFMT ", repfnz_col[" IFMT "] = " IFMT "\n",
-			jj, k, repfnz[(jj-jcol)*n + k]);
+		fprintf(stderr, "col %d, repfnz_col[%d] = %d\n", jj,
+			k, repfnz[(jj-jcol)*n + k]);
 		ABORT("check_repfnz_dist");
 	    }
 }
@@ -777,18 +742,6 @@ void PrintInt10(char *name, int_t len, int_t *x)
 {
     register int_t i;
     
-    printf("%10s:", name);
-    for (i = 0; i < len; ++i) {
-	if ( i % 10 == 0 ) printf("\n\t[" IFMT "-" IFMT "]", i, i+9);
-	printf(IFMT, x[i]);
-    }
-    printf("\n");
-}
-
-void PrintInt32(char *name, int len, int *x)
-{
-    register int i;
-    
     printf("%10s:", name);
     for (i = 0; i < len; ++i) {
 	if ( i % 10 == 0 ) printf("\n\t[%2d-%2d]", i, i+9);
@@ -801,26 +754,12 @@ int file_PrintInt10(FILE *fp, char *name, int_t len, int_t *x)
 {
     register int_t i;
     
-    fprintf(fp, "%10s:", name);
-    for (i = 0; i < len; ++i) {
-	if ( i % 10 == 0 ) fprintf(fp, "\n\t[" IFMT "-" IFMT "]", i, i+9);
-	fprintf(fp, IFMT, x[i]);
-    }
-    fprintf(fp, "\n");
-    return 0;
-}
-
-int file_PrintInt32(FILE *fp, char *name, int len, int *x)
-{
-    register int i;
-    
     fprintf(fp, "%10s:", name);
     for (i = 0; i < len; ++i) {
 	if ( i % 10 == 0 ) fprintf(fp, "\n\t[%2d-%2d]", i, i+9);
 	fprintf(fp, "%6d", x[i]);
     }
     fprintf(fp, "\n");
-    return 0;
 }
 
 int_t
@@ -845,105 +784,3 @@ CheckZeroDiagonal(int_t n, int_t *rowind, int_t *colbeg, int_t *colcnt)
     return numzd;
 }
 
-
-/* --------------------------------------------------------------------------- */
-void isort(int_t N, int_t *ARRAY1, int_t *ARRAY2)
-{
-/*
- * Purpose
- * =======
- * Use quick sort algorithm to sort ARRAY1 and ARRAY2 in the increasing
- * order of ARRAY1.
- *
- * Arguments
- * =========
- * N       (input) INTEGER
- *          On entry, specifies the size of the arrays.
- *
- * ARRAY1  (input/output) DOUBLE PRECISION ARRAY of LENGTH N
- *          On entry, contains the array to be sorted.
- *          On exit, contains the sorted array.
- *
- * ARRAY2  (input/output) DOUBLE PRECISION ARRAY of LENGTH N
- *          On entry, contains the array to be sorted.
- *          On exit, contains the sorted array.
- */
-  int_t IGAP, I, J;
-  int_t TEMP;
-  IGAP = N / 2;
-  while (IGAP > 0) {
-    for (I = IGAP; I < N; I++) {
-    J = I - IGAP;
-    while (J >= 0) {
-      if (ARRAY1[J] > ARRAY1[J + IGAP]) {
-        TEMP = ARRAY1[J];
-        ARRAY1[J] = ARRAY1[J + IGAP];
-        ARRAY1[J + IGAP] = TEMP;
-        TEMP = ARRAY2[J];
-        ARRAY2[J] = ARRAY2[J + IGAP];
-        ARRAY2[J + IGAP] = TEMP;
-        J = J - IGAP;
-      } else {
-        break;
-      }
-    }
-  }
-    IGAP = IGAP / 2;
-  }
-}
-
-
-void isort1(int_t N, int_t *ARRAY)
-{
-/*
- * Purpose
- * =======
- * Use quick sort algorithm to sort ARRAY1 and ARRAY2 in the increasing
- * order of ARRAY1.
- *
- * Arguments
- * =========
- * N       (input) INTEGER
- *          On entry, specifies the size of the arrays.
- *
- * ARRAY1  (input/output) DOUBLE PRECISION ARRAY of LENGTH N
- *          On entry, contains the array to be sorted.
- *          On exit, contains the sorted array.
- *
- * ARRAY2  (input/output) DOUBLE PRECISION ARRAY of LENGTH N
- *          On entry, contains the array to be sorted.
- *          On exit, contains the sorted array.
- */
-  int_t IGAP, I, J;
-  int_t TEMP;
-  IGAP = N / 2;
-  while (IGAP > 0) {
-  for (I = IGAP; I < N; I++) {
-    J = I - IGAP;
-    while (J >= 0) {
-      if (ARRAY[J] > ARRAY[J + IGAP]) {
-        TEMP = ARRAY[J];
-        ARRAY[J] = ARRAY[J + IGAP];
-        ARRAY[J + IGAP] = TEMP;
-        J = J - IGAP;
-      } else {
-        break;
-      }
-    }
-  }
-    IGAP = IGAP / 2;
-  }
-}
-
-void log_memory(long long cur_bytes, SuperLUStat_t *stat) {
-    stat->current_buffer += (float) cur_bytes;
-    if (cur_bytes > 0) {
-	stat->peak_buffer = 
-	    SUPERLU_MAX(stat->peak_buffer, stat->current_buffer);
-    }
-}
-
-void print_memorylog(SuperLUStat_t *stat, char *msg) {
-    printf("__ %s (MB):\n\tcurrent_buffer : %8.2f\tpeak_buffer : %8.2f\n",
-	   msg, stat->current_buffer, stat->peak_buffer);
-}
