@@ -2,6 +2,7 @@
 from .ast import *
 from .obj import *
 from .obj import _LineElement
+from .model import Node
 
 Dof = Int
 
@@ -9,18 +10,14 @@ def Yng(about=None, **kwds):
     about = about or "Young's modulus of elasticity"
     return Num("E", field="elastic_modulus", about=about, **kwds)
 
+def Yld(nature, about=None, **kwds):
+    about = about or f"Yield {nature}."
+    return Num("fy", field=f"yield_{nature}", about=about, **kwds)
+
 def Area(**kwds):
     return Num("A", field="area", about="cross-sectional area", **kwds)
 
 
-Node = cmd("Node", "node", [
-    Tag(),
-    Grp("crd", type=Num, args=[Num("x"), Num("y"), Num("z")]),
-    Grp("mass", flag="-mass", reqd=False, default=[0.0]*6, args=[
-        Num("x"),Num("y"), Num("z"), Num("x"),Num("y"), Num("z")
-    ]),
-])
-
 class BeamInt(Arg):
     def as_tcl_list(self, value):
         args = []
@@ -56,6 +53,35 @@ class uniaxial:
     A `UniaxialMaterial` object typically represents a pair of work conjugate
     scalars such as axial stress/strain, moment/cuvature, or force/deformation.
     """
+    class UniaxialWrapper:
+        def __init__(self, wrapped):
+            self._wrapped = wrapped
+            self._wrapping_several = isinstance(self._wrapped, (tuple, list))
+
+            
+        def __repr__(self):
+            return f"{self.__class__.__name__}<{self._wrapped.__class__.__name__}>"
+
+        def __enter__(self):
+            if self._wrapping_several:
+                self.wrapped = type(self._wrapped)(
+                    w.__enter__() for w in self._wrapped
+                )
+            else:
+                self.wrapped = self._wrapped.__enter__()
+
+            return self
+
+        def __exit__(self, *args, **kwds):
+            if self._wrapping_several:
+                for i,w in enumerate(self._wrapped):
+                    w.__exit__(*args, **kwds)
+                    del self.wrapped[i]
+            else:
+                self._wrapped.__exit__(*args, **kwds)
+            del self.wrapped
+
+
     Series = Uni("SeriesMaterial", "Series", args = [
             Tag(),
             Grp("materials", min=2, type=Ref(type="uniaxial", attr="name"))
@@ -84,19 +110,27 @@ class uniaxial:
 
     ElasticBilin = Uni("ElasticBilin", "ElasticBilin", args=[ 
         Tag(),
-        Yng(about="tangent in tension for stains: 0 <= strains <= $epsP2"),
-        Num("E2", about="tangent when material in tension with strains > $epsP2"),
+        Yng(about=r"tangent in tension for stains: 0 <= strains $\le$ `epsP2`"),
+        Num("E2", about="tangent when material in tension with strains > `epsP2`"),
         Num("eps2", about="strain at which material changes tangent in tension."),
-        Num("EN1", reqd=False, about="optional, default = $EP1. tangent in compression for stains: 0 < strains <= $epsN2"),
-        Num("EN2", reqd=False, about="optional, default = $EP2. tangent in compression with strains < $epsN2"),
+        Num("EN1", reqd=False, about="optional, default = `EP1`. tangent in compression for stains: 0 < strains $\le$ `epsN2`"),
+        Num("EN2", reqd=False, about="optional, default = `EP2`. tangent in compression with strains < `epsN2`"),
         Num("epsN2", reqd=False, about="optional, default = -epsP2. strain at which material changes tangent in compression.")
     ])
 
+    ElasticMultilinear = Uni("ElasticMultilinear", "ElasticMultilinear", args=[
+            Tag(),
+            Grp("points", type=Grp, args=[Grp(type=Num, args=[Num("-strain"), Num("-stress")])],
+                about="Points defining multilinear backbone."
+            ),
+        ]
+    )
+
 
     Hardening = Uni("HardeningMaterial", "Hardening", args=[
         Tag(),
         Yng(),
-        Num("fy", about="yield stress or force"),
+        Yld("stress"),
         Num("H_iso", about="isotropic hardening Modulus"),
         Num("H_kin", about="kinematic hardening Modulus"),
         Num("eta",   reqd=False, about="visco-plastic coefficient (optional, default=0.0)"),
@@ -130,20 +164,21 @@ class uniaxial:
         ])
 
     RambergOsgoodSteel = Uni("RambergOsgoodSteel", "RambergOsgoodSteel", args=[
-        Tag(), Yng(), Num("fy", about="yield strength"), Num("a", about="yield offset parameter"),
-        Num("n", about="")
+        Tag(), Yng(), Yld("stress"), Num("a", about="yield offset parameter"),
+        Num("n", about="Parameter to control the transition from elastic to plastic branches. Additionally controls the hardening of the material: by increasing $n$, hardening ratio will be decreased.")
     ])
 
     DoddRestrepo = Uni("Dodd_Restrepo", "Dodd_Restrepo", args=[
         Tag(),
-        Num("Fy",  about="Yield strength"),
-        Num("Fsu", about="Ultimate tensile strength (UTS)"),
+        Yld("stress"),
+        Num("fu",  about="Ultimate tensile strength (UTS)"),
         Num("esh", about="Tensile strain at initiation of strain hardening"),
         Num("esu", about="Tensile strain at the UTS"),
         Yng(),
-        Num("eshI", about="Tensile strain for a point on strain hardening curve, recommended range of values for eshI: [ (ESU + 5esh)/6, (ESU + 3esh)/4]"),
+        Num("eshI", about="Tensile strain for a point on strain hardening curve, recommended range of values for eshI: " \
+                r"$\left\[ (\texttt{esu} + 5 \texttt{esh})/6, (\texttt{esu} + 3 \texttt{esh})/4\right\]$"),
         Num("fshI", about="Tensile stress at point on strain hardening curve corresponding to eshI"),
-        Num("OmegaFac", reqd=False, about="Roundedness factor for Bauschinger curve in cycle reversals from the strain hardening curve. Range: [0.75, 1.15]. Largest value tends to near a bilinear Bauschinger curve. Default = 1.0."),
+        Num("OmegaFac", default=1.0, reqd=False, about=r"Roundedness factor for Bauschinger curve in cycle reversals from the strain hardening curve. Range: $\left[0.75, 1.15\right]$. Largest value tends to near a bilinear Bauschinger curve. Default = 1.0."),
     ])
 
 
@@ -178,6 +213,126 @@ class uniaxial:
         # Recommended values: R0=between 10 and 20, cR1=0.925, cR2=0.15"),
     )
 
+    ReinforcingSteel = Uni("ReinforcingSteel", "ReinforcingSteel", args=[
+        Num("matTag", about="unique material object integer tag"),
+        Yld("stress", about="Yield stress in tension (see Figure 1)"),
+        Num("fu",     about="Ultimate stress in tension"),
+        Yng(          about="Initial elastic tangent"),
+        Num("Esh",    about="Tangent at initial strain hardening"),
+        Num("esh",    about="Strain corresponding to initial strain hardening"),
+        Num("eult",   about="Strain at peak stress"),
+        Grp("-GABuck", about="Buckling Model Based on Gomes and Appleton (1997)", args=[
+            Num("lsr", about="Slenderness Ratio (see Figure 2)"),
+            Num("beta", about="Amplification factor for the buckled stress strain curve. (see Figure 3)"),
+            Num("r", about="""Buckling reduction factor
+                r can be a real number between [0.0 and 1.0]
+                r=1.0 full reduction (no buckling)
+                r=0.0 no reduction
+                0.0<r<1.0 linear interpolation between buckled and unbuckled curves
+                """)
+            ]
+          ),
+        Num("gamma", about="Buckling constant (see Figures 3 and 4)"),
+
+        Grp("-DMBuck", about="Buckling model based on Dhakal and Maekawa (2002)", args=[
+            Num("lsr", about="Slenderness Ratio (see Figure 2)"),
+            Num("alpha", about="Adjustment Constant usually between 0.75 and 1.0", default=1.0),
+          ]
+        ),
+
+        Grp("-CMFatigue", about="Coffin-Manson Fatigue and Strength Reduction", args=[
+            Num("Cf", about="Coffin-Manson constant C (see Figure 5)"),
+            Num("alpha", about="Coffin-Manson constant a (see Figure 5)"),
+            Num("Cd", about="Cyclic strength reduction constant (see Figure 6 and Equation 3)"),
+          ]
+        ),
+
+        Grp("-IsoHard", about="Isotropic Hardening / Diminishing Yield Plateau", args=[
+            Num("a1", about="Hardening constant", default=4.3),
+            Num("limit", about="Limit for the reduction of the yield plateau. % of original plateau length to remain (0.01 < limit < 1.0 ). If `limit` = 1.0, then no reduction takes place", default=0.01),
+          ]
+        ),
+
+        Grp("-MPCurveParams", about="Menegotto and Pinto Curve Parameters see Fig 6b", args=[
+            Num("R1", default = 0.333),
+            Num("R2", default = 18),
+            Num("R3", default = 4)
+          ]
+        )
+    ])
+    Steel04 = Uni("Steel04", "Steel04", args=[
+        
+        Tag(),
+        Yld("stress"),
+        Yng(),
+        Grp("-kin", type=Num, about="""apply kinematic hardening
+            Kinematic hardening is based on the Menegotto-Pinto model. The parameters and their use is identical to those of the Steel02 material.
+
+            Steel4 param kin.png
+
+            recommended values: $R_0 = 20 $r_1 = 0.90 $r_2 = 0.15""", args=[
+            Num("b_k", about="hardening ratio (E_k/E_0)"),
+            Num("R_0", about="control the exponential transition from linear elastic to hardening asymptote"),
+            Num("r_1"),
+            Num("r_2"),
+        ]),
+
+        Grp("-iso", about="""apply isotropic hardening
+            Isotropic hardening increases the yield strength of the material. The applied increase is calculated as a function of the accumulated plastic strain. The following parameters control that function.
+
+            Steel4 param iso.png""", args=[
+            Num("b_i", about="initial hardening ratio (E_i/E_0)"),
+            Num("b_l", about="saturated hardening ratio (E_is/E_0)"),
+            Num("rho_i", about="specifies the position of the intersection point between initial and saturated hardening asymptotes"),
+            Num("R_i", about="control the exponential transition from initial to saturated asymptote"),
+            Num("l_yp", about=r"length of the yield plateau in $\varepsilon_{y0} = f_y / E_0$ units"),
+        ]),
+
+        Grp("-ult", type=Num, about="""apply an ultimate strength limit
+            The ultimate strength limit serves as an upper limit of material resistance. After the limit is reached the material behaves in a perfectly plastic manner. Exponential transition is provided from the kinematic hardening to the perfectly plastic asymptote.
+            Note that isotropic hardening is also limited by the ultimate strength, but the transition from the isotropic hardening to the perfectly plastic asymptote is instantaneous.
+            Steel4 param ult.png
+            """, args=[
+                Num("f_u", about="ultimate strength"),
+                Num("R_u", about="control the exponential transition from kinematic hardening to perfectly plastic asymptote"),
+        ]),
+
+        Grp("-asym", reqd=False, type=Grp, about="""assume non-symmetric behavior
+                If non-symmetric behavior is assumed, material response under tension and compression will be controlled by two different parameter sets. The normal parameters control behavior under tension. Additional parameters shall be specified to describe behavior under compression. The following parameters are expected after the normal parameters when the options below are used.
+                Steel4 param asymi.png
+                Steel4 param asymk.png""", args=[
+                Grp("-kin", type=Num, num=4, args=[Num("b_kc"), Num("R_0c"), Num("r_1c"), Num("r_2c")]),
+                Grp("-iso", type=Num, num=4, args=[Num("b_ic"), Num("rho_ic"), Num("b_lc"), Num("R_ic")]),
+                Grp("-ult", type=Num, num=2, args=[Num("f_uc"), Num("R_uc")]),
+            ]
+        ),
+
+        Num("sig_init", flag="-init", about="""apply initial stress
+            Initial stress is assumed at 0 strain at the beginning of the loading process. The absolute value of the initial stress is assumed to be less than the yield strength of the material.
+            Steel4 param init.png"""
+        ),
+
+        Int("cycNum", flag="-mem", about="""expected number of half-cycles during the loading process
+            Efficiency of the material can be slightly increased by correctly setting this value. The default value is $cycNum = 50
+            Load history memory can be turned off by setting $cycNum = 0."""
+        ),
+
+    ])
+
+    UVCuniaxial = UVC = Uni("UVCuniaxial", "UVCuniaxial", args = [
+        Tag(),
+        Yng(),
+        Yld("stress"),
+        Num("QInf",   about="Maximum increase in yield stress due to cyclic hardening (isotropic hardening)."),
+        Num("b",      about="Saturation rate of QInf, b > 0."),
+        Num("DInf",   about="Decrease in the initial yield stress, to neglect the model updates set DInf = 0."),
+        Num("a",      about="Saturation rate of DInf, $a > 0$. If $D_\infty == 0$, then a is arbitrary (but still a > 0)."),
+        Int("N",      about=r"Number of backstresses to define, $N \ge 1$."),
+        Grp("backstress", type=Grp, args=[Grp(type=Num, args=[Num("C"), Num("gamma")])],
+            about="Backstress parameters, up to 9 pairs may be specified. If `C` is specified, then the corresponding `gamma` must also be specified. Note that only the first N backstresses will be read by the parser."
+        ),
+    ])
+
     ConfinedConcrete01  = Uni("ConfinedConcrete01", "ConfinedConcrete01", args=[
          Tag(),
          Str("secType", enum={
@@ -270,7 +425,7 @@ class uniaxial:
         Num("ec"  ,  about="floating point values defining concrete strain at maximum strength*"),
         Num("ecu" ,  about="floating point values defining concrete strain at crushing strength*"),
         Num("Ec"  ,  about="floating point values defining initial stiffness**"),
-        Grp("tension", reqd=False, typ=Num, args=[
+        Grp("tension", reqd=False, type=Num, args=[
             Num("fct" ,  about="floating point value defining the maximum tensile strength of concrete"),
             Num("et"  ,  about="floating point value defining ultimate tensile strain of concrete"),
         ]),
@@ -279,28 +434,42 @@ class uniaxial:
 
     Concrete02IS = Uni("Concrete02IS", "Concrete02IS", [
          Tag(), 
-         Num("E0"), 
+         Yng(), 
          Num("fpc"), 
          Num("epsc0"), 
          Num("fpcu"), 
          Num("epscu"), 
-         Grp("tension", reqd=False, typ=Num, args=[
+         Grp("tension", reqd=False, type=Num, args=[
             Num("rat"), Num("ft"), Num("Ets")
          ])
     ])
 
+    Concrete06 = Uni("Concrete06", "Concrete06", args=[ 
+        Tag("matTag", about="integer tag identifying material"),
+        Num("fc", about="concrete compressive strength (compression is negative)*"),
+        Num("e0", about="strain at compressive strength*"),
+        Num("n", about="compressive shape factor"),
+        Num("k", about="post-peak compressive shape factor"),
+        Num("alpha1", about=r"$\alpha_1$ parameter for compressive plastic strain definition"),
+        Num("fcr", about="tensile strength"),
+        Num("ecr", about="tensile strain at peak stress (`fcr`)"),
+        Num("b", about="exponent of the tension stiffening curve"),
+        Num("alpha2", about=r"$\alpha_2$ parameter for tensile plastic strain definition"),
+    ])
+
+
     ConcreteCM = Uni("ConcreteCM", "ConcreteCM", args=[ 
         Tag(), #"mattag  Unique uniaxialMaterial tag"),
-        Num("fpcc" , about="Compressive strength (f'c)"),
-        Num("epcc" , about="Strain at compressive strength (<math>\epsilon</math>'c)"),
-        Num("Ec"   , about="Initial tangent modulus (Ec)"),
-        Num("rc"   , about="Shape parameter in Tsai’s equation defined for compression (rc)"),
-        Num("xcrn" , about="Non-dimensional critical strain on compression envelope (<math>\epsilon</math>-cr, where the envelope curve starts following a straight line)"),
-        Num("ft"   , about="Tensile strength (ft)"),
-        Num("et"   , about="Strain at tensile strength (<math>\epsilon</math>t)"),
-        Num("rt"   , about="Shape parameter in Tsai’s equation defined for tension (rt)"),
-        Num("xcrp" , about="Non-dimensional critical strain on tension envelope (<math>\epsilon</math>+cr, where the envelope curve starts following a straight line – large value [e.g., 10000] recommended when tension stiffening is considered)"),
-        Int("gap", reqd=False, flag="-GapClose",  about="gap = 0, less gradual gap closure (default); gap = 1, more gradual gap closure"),
+        Num("fpcc" , about=r"Compressive strength (f'c)"),
+        Num("epcc" , about=r"Strain at compressive strength ($\epsilon^\prime_c$)"),
+        Num("Ec"   , about=r"Initial tangent modulus ($E_c$)"),
+        Num("rc"   , about=r"Shape parameter in Tsai’s equation defined for compression ($r_c$)"),
+        Num("xcrn" , about=r"Non-dimensional critical strain on compression envelope ($\epsilon^-_{cr}$, where the envelope curve starts following a straight line)"),
+        Num("ft"   , about=r"Tensile strength ($f_t$)"),
+        Num("et"   , about=r"Strain at tensile strength ($\epsilon$t)"),
+        Num("rt"   , about=r"Shape parameter in Tsai’s equation defined for tension ($r_t$)"),
+        Num("xcrp" , about=r"Non-dimensional critical strain on tension envelope ($\epsilon^+_{cr}$, where the envelope curve starts following a straight line – large value [e.g., 10000] recommended when tension stiffening is considered)"),
+        Int("gap",   reqd=False, flag="-GapClose",  about="gap = 0, less gradual gap closure (default); gap = 1, more gradual gap closure"),
     ])
 
 
@@ -364,12 +533,12 @@ class element:
         _args = [
                 
             Tag(),
-            Grp("nodes", typ=Node, 
+            Grp("nodes", type=Node, 
                 args=[Ref("iNode", attr="name"), Ref("jNode", attr="name")], about="end nodes"),
             Num("A", about="cross-sectional area of element"),
-            Ref("material", attr="name", typ=Uni, about="tag associated with previously-defined UniaxialMaterial"),
-            #Ref("section", typ="Section", about="tag associated with previously-defined Section"),
-            Num("rho", reqd=False, about="mass per unit length, optional, default = 0.0"),
+            Ref("material", attr="name", type=Uni, about="tag associated with previously-defined UniaxialMaterial"),
+            #Ref("section", type="Section", about="tag associated with previously-defined Section"),
+            Num("rho",   reqd=False, about="mass per unit length, optional, default = 0.0"),
             Int("cFlag", reqd=False, about="consistent mass flag, optional, default = 0"+
                 "cFlag = 0 lumped mass matrix (default)"+
                 "cFlag = 1 consistent mass matrix"),
@@ -455,8 +624,7 @@ class element:
         inherit=[_LineElement],
     )
 
-LinearTransform = Trf("LinearTransform",
-  "Linear",
+LinearTransform = Trf("LinearTransform", "Linear",
   args = [
     Tag("name", about="Tag used to identify the transform object."),
     Grp("vecxz", type=Num, num=3),
@@ -481,9 +649,7 @@ Backbone = LibCmd("backbone")
 class backbone:
 
     Popovics = Mander = Backbone("Mander",
-        args = [
-          Tag(), Num("fc"), Num("epsc"), Num("Ec")
-        ]
+        args = [Tag(), Num("fc"), Num("epsc"), Yng()]
     )
 
     ReeseSoftClay = Backbone("ReeseSoftClay",
@@ -506,14 +672,12 @@ class backbone:
 
     Raynor = Backbone("Raynor", 
         args=[
-          Tag(), Num("Es"), Num("fy"), Num("fsu"), Num("Epsilonsh"), Num("Epsilonsm"), Num("C1"), Num("Ey")
+          Tag(), Yng(), Yld("stress"), Num("fsu"), Num("Epsilonsh"), Num("Epsilonsm"), Num("C1"), Num("Ey")
         ]
     )
 
     Backbone("Capped", 
-        args=[
-          Tag(), Ref("backbone"), Num("capTag")
-        ]
+        args=[Tag(), Ref("backbone"), Num("capTag")]
     )
 
     LinearCapped = Backbone("LinearCapped", 
@@ -522,6 +686,13 @@ class backbone:
         ]
     )
 
+    Multilinear = Backbone("Multilinear", "Multilinear", args=[
+            Grp("points", type=Grp, args=[Grp(type=Num, args=[Num("e"), Num("s")])],
+                about="Points defining multilinear backbone."
+            ),
+        ]
+    )
+
 
 # region = Cmd("region", about="""
 #     The region command is used to label a group of nodes and elements. This command
